import os,datetime,glob
import numpy as np
import matplotlib as mpl
mpl.use("Agg")
from windrose import WindroseAxes
from matplotlib import pyplot as plt
import pycdf

ncDir = '/seacoos/data/nccoos/level1/billymitchell/sodar1' 
ncFileGlob = 'billymitchell_sfas_*.nc' 
 
pngDir = '/seacoos/data/nccoos/level1/billymitchell/sodar1/plots/winddist'
if not os.path.exists(pngDir):
    os.mkdir(pngDir)

pngExt = 'png' 
htmlExt = 'html' 
 
ncFilePattern = os.path.join(ncDir,ncFileGlob) 
files = glob.glob(ncFilePattern)
files = files[:-1] # sodar was broken last month
previous = [files[-1]] + files[:-1] 
next = files[1:] + [files[0]] 
files = zip(previous,files,next) 

html1 = """<html> 
    <head> 
        <title> 
            Billy Mitchell Sodar :: """


html2 = """ :: Wind Distribution at """

html3 = """ Meters Above Sea Level
        </title> 
    </head> 
    <body>
        <form> 
            <a href=\""""

html4 = """\">&lt;Previous Month</a>&nbsp;&nbsp;&nbsp; 
            <a href=\""""

html5 = """\">Next Month&gt;</a>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
            <a href=\""""

html6 = """\">&lt;Previous Elevation</a>&nbsp;&nbsp;&nbsp; 
            <a href=\""""

html7 = """\">Next Elevation&gt;</a>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
            <a href=\"all.html\">&lt;All At Once&gt;</a>
        </form>
        <br/> 
        <img src=\""""

html8 = """\"/> 
    </body> 
</html>"""
 
firstMonth = True 
for previous,ncFile,next in files: 
    print 'Processing',ncFile 
    ncFileName = os.path.splitext(os.path.basename(ncFile))[0] 
    year = ncFileName[19:23] 
    month = ncFileName[24:26] 
    month = datetime.datetime(int(year),int(month),1).strftime('%B') 
    if previous: 
        previous = os.path.splitext(os.path.basename(previous))[0] 
        previousYear = previous[19:23] 
        previousMonth = previous[24:26] 
        previousMonth = datetime.datetime(int(previousYear),int(previousMonth),1).strftime('%B') 
    if next: 
        next = os.path.splitext(os.path.basename(next))[0] 
        nextYear = next[19:23] 
        nextMonth = next[24:26] 
        nextMonth = datetime.datetime(int(nextYear),int(nextMonth),1).strftime('%B') 
 
    nc = pycdf.CDF(ncFile) 

    t = nc.var('time')[:] 
    z = nc.var('z')[:] 
    u = nc.var('u')[:] 
    v = nc.var('v')[:] 
    w = nc.var('w')[:] 
 
    mu = np.ma.masked_invalid(u) 
    mv = np.ma.masked_invalid(v) 
    mw = np.ma.masked_invalid(w) 
 
    mumean = mu.mean(axis=0)
    mvmean = mv.mean(axis=0)
    mwmean = mw.mean(axis=0)
 
    ucount = mu.count(axis=0)
    vcount = mv.count(axis=0)
    wcount = mw.count(axis=0)

    mrho = np.sqrt((mu**2) + (mv**2))
    mrho = mrho.T
    mtheta = (180 * np.arctan2(mv, mu)/np.pi)
    theta = np.piecewise(mtheta,
                         (mtheta <= 90, mtheta > 90),
                         (lambda x: 90 - x, lambda x: 450 - x))
    mtheta = np.ma.array(theta,mask=mtheta.mask)
    mtheta = mtheta.T

    rho = [np.array([elem
                     for elem,mask
                     in zip(row.data,row.mask)
                     if not mask])
           for row
           in mrho]
    theta = [np.array([elem
                       for elem,mask
                       in zip(row.data,row.mask)
                       if not mask])
             for row
             in mtheta]

    firstElevation = True
    indices = range(len(rho))
    previousX = [indices[-1]] + indices[:-1] 
    nextX = indices[1:] + [indices[0]] 
    indices = zip(previousX,indices,nextX) 
    for previousX,x,nextX in indices:
        if rho[x].any():
            fig = plt.figure(figsize=(8, 8), dpi=80, facecolor='w', edgecolor='w')
            spt = fig.suptitle('Billy Mitchell Sodar :: ' + month + " " + year)
            rect = [0.16, 0.16, 0.68, 0.68]
            ax = WindroseAxes(fig, rect, axisbg='w')
            ax.set_title('Wind Distribution at %d Meters Above Sea Level\nPercentage of Wind Magnitude From Direction\n\n' % z[x],fontsize=spt.get_fontsize()*0.8)
            fig.add_axes(ax)
            ax.bar(theta[x], rho[x], bins = np.arange(0.0,20.0,2.5), normed=True, opening=0.8, edgecolor='white')
            yticklabels = ['',] + \
                          [plt.getp(textobj,'text')+"%"
                               for textobj in plt.getp(ax,'yticklabels')[1:]]
            plt.setp(ax,yticklabels=yticklabels)
            l = ax.legend(axespad=-0.20,title="Magnitude (m/s)")
            plt.setp(l.get_texts(), fontsize=8)
            if not os.path.exists(os.path.join(pngDir,ncFileName)):
                os.mkdir(os.path.join(pngDir,ncFileName))
            outFile = os.path.join(pngDir,ncFileName,('%dm' % z[x]) + os.extsep + pngExt) 
            print 'Saving', outFile 
            fig.savefig(outFile)

            # htmlFile = os.path.join(pngDir,ncFileName + "_" + ('%dm' % z[x]) + os.extsep + htmlExt) 
            # html = html1 + month + " " + year
            # html = html + html2 + ('%dm' % z[x])
            # html = html + html3 + previous + "_" + ('%dm' % z[x]) + os.extsep + htmlExt
            # html = html + html4 + next + "_" + ('%dm' % z[x]) + os.extsep + htmlExt
            # html = html + html5 + ncFileName + "_" + ('%dm' % z[previousX]) + os.extsep + htmlExt
            # html = html + html6 + ncFileName + "_" + ('%dm' % z[nextX]) + os.extsep + htmlExt
            # html = html + html7 + os.path.join(ncFileName,os.path.basename(outFile)) + html8 
            # handle = open(htmlFile,'w') 
            # handle.write(html) 
            # handle.close() 

            # if firstMonth and firstElevation: 
            #     firstMonth = False
            #     firstElevation = False
            #     htmlFile = os.path.join(pngDir,"index" + os.extsep + htmlExt) 
            #     handle = open(htmlFile,'w') 
            #     handle.write(html) 
            #     handle.close() 
 
